
*:**************************************************************************
*:
*:      Procedure: ValidFName
*:      
*:  Last Modified: October 20, 1993  3:09pm
*:
*:        Purpose: This procedure is used to validate a filename, and
*:                 return an error message, if appropriate.
*:
*:         Author: Michael P. Lascuola  CIS [72271,723]
*:         
*:     Parameters: lcFileName - Filename to check
*:               : lcErrorMsg - Error message returned if invalid filename
*:               
*:        Example: ...
*:                 
*:                 lcName = SPACE(80)
*:                 
*:                 @ 3, 0 SAY "Enter filename: " GET lcName SIZE 1, 31
*:                 
*:                 lcMessage = ""
*:                 
*:                 * Pass lcMessage by Reference, to allow
*:                 * ValidFName to modify it
*:                 llValid = ValidFName(lcName, @lcMessage)
*:                 IF NOT llValid
*:                    WAIT WINDOW "Invalid Filename " + lcMessage
*:                 ENDIF
*:                 
*:                 ...
*:        
*:
*:**************************************************************************
PROCEDURE ValidFName
PARAMETERS lcFileName, lcErrorMsg

PRIVATE lcErrorMsg    && String to hold error message, if any
PRIVATE lnPathPos     && Position of the path in the filename, if any
PUBLIC  lnChar        && ASC() value of characters in filename
PRIVATE lcPath        && Path specified in lcFileName, if any
PRIVATE lnPeriod      && Position of the period in the filename, if any
PRIVATE lcStem        && Part of the filename to the left of the period
PRIVATE lcExt         && Part of the filename to the right of the period

lcErrorMsg = ""
lcFileName = UPPER(lcFileName)

* ķ
*  Check to see if filename is a directory.                         
* Ľ
lnPathPos = RAT("\", lcFileName)

* ķ
*  If Directory exists parse out filename & verify directory.       
* Ľ
IF lnPathPos > 0
   lcPath     = SUBSTR(lcFileName, 1, lnPathPos)
   lcFileName = SUBSTR(RTRIM(lcFileName), lnPathPos + 1)
   IF NOT ValidDir(lcPath)
      lcErrorMsg = " Invalid directory specified. "
      RETURN .F.
   ENDIF
ELSE
   lcPath  = ""
   lcFileName = RTRIM(lcFileName)
ENDIF

* ķ
*  Check for spaces within the filename.                            
* Ľ
IF " " $ lcFileName
   lcErrorMsg = " Filenames cannot have spaces. "
   RETURN .F.
ENDIF

* ķ
*  Allow for an empty filename, in case this procedure is in a      
*  VALID clause and a user is tabbing to another field.             
* Ľ
IF LEN(ALLTRIM(lcFileName)) = 0
   RETURN .T.
ENDIF

* ķ
*  If there is an extension, the file stem may contain up to 8      
*  characters.                                                      
* Ľ
lnPeriod = AT ( "." , lcFileName)

IF lnPeriod > 9
   lcErrorMsg = " Filenames must be no longer than 8 " + ;
      "characters plus a 3 character extension. "
   RETURN .F.
ENDIF

* ķ
*  Check to see if there are more than 1 period in the filename.    
* Ľ
IF OCCURS (".", lcFileName) > 1
   lcErrorMsg = ' Only 1 "." is allowed to separate ' + ;
      'filename stem from the extension. '
   RETURN .F.
ENDIF

IF lnPeriod > 0
   lcStem = SUBSTR(lcFileName, 1, lnPeriod - 1)
   lcExt = RTRIM(SUBSTR(lcFileName, lnPeriod + 1))
   IF LEN(lcExt) > 3
      lcErrorMsg = " Filename extensions must be no longer " + ;
         "than 3 characters. "
      RETURN .F.
   ENDIF
   IF LEN(lcStem) = 0
      lcErrorMsg = " The stem of a filename must be at least 1 character. "
      RETURN .F.
   ENDIF
ELSE
   lcStem = RTRIM(lcFileName)
   lcExt = ""

   IF LEN(lcStem) > 8
      lcErrorMsg = " Filenames must be no longer than 8 characters. "
      RETURN .F.
   ENDIF
ENDIF

* ķ
*  Check for DOS' reserved filenames.                               
* Ľ
DO CASE
CASE LEN(lcStem) = 3
   IF INLIST (lcStem, "CON", "AUX", "NUL" , "PRN")
      lcErrorMsg =lcStem + " is a DOS reserved filename. "
      RETURN .F.
   ENDIF
   
CASE LEN(lcStem) = 4
   IF INLIST (lcStem, "COM1", "COM2", "COM3", "COM4",  ;
         "LPT1", "LPT2", "LPT3")
      lcErrorMsg =lcStem + " is a DOS reserved filename. "
      RETURN .F.
   ENDIF
   
OTHERWISE
   IF lcStem == "CLOCK$"
      lcErrorMsg =lcStem + " is a DOS reserved filename. "
      RETURN .F.
   ENDIF
ENDCASE

* ķ
*  Last Check:  validate each remaining character.  If no character 
*  is invalid, return .T.                                           
* Ľ
IF ValidChars(lcStem)
   IF ValidChars(lcExt)
      RETURN .T.
   ELSE
      lcErrorMsg = '"' + CHR(lnChar) + '" is an invalid ' + ;
         'filename extension character. '
      RETURN .F.
   ENDIF
ELSE
   lcErrorMsg = '"' + CHR(lnChar) + '" is an invalid ' + ;
      'filename character. '
   RETURN .F.
ENDIF

* End of PROCEDURE ValidFName *



*!**************************************************************************
*!
*!       Function: ValidChars()
*!
*!        Purpose: Checks each character in a passed string to make sure
*!                 it can be part of a valid filename.             
*!
*!**************************************************************************
FUNCTION ValidChars
PARAMETERS lcString

PRIVATE lnCount        && Loop counter

* ķ
*  Start from the right of the string in case someone enters a      
*  forward slash. (c:/dos/myfile.txt                                
* Ľ
FOR lnCount = 0 TO LEN(lcString) - 1
   lnChar = ASC(SUBSTR(lcString, LEN(lcString) - lnCount, 1))
   
   * ķ
   *  Check for alpha or numeric character.                         
   * Ľ
   IF (lnChar >= 65 AND lnChar <= 90)  OR ;
      (lnChar >= 48 AND lnChar <= 57)  OR ;
       lnChar == 45 OR  lnChar == 95
      LOOP
   ENDIF
   
   * ķ
   *  Check for valid special characters.                           
   * Ľ
   IF OCCURS (CHR(lnChar), "_^$~!#%&-{}()@'.") > 0
      LOOP
   ENDIF

   * ķ
   *  If still in loop, string contains an invalid character.       
   * Ľ
   RETURN .F.
ENDFOR

RETURN .T.


*!*****************************************************************************
*!
*!       Function: ValidDir() [Taken from FoxPro Goodies IsDir()]
*!
*!        Purpose: Checks passed string to make sure is is a vaild directory
*!
*!*****************************************************************************
FUNCTION ValidDir
PARAMETER lcString

PRIVATE lnHandle
IF RIGHT(lcString, 1) <> "\"
   lcString = lcString + "\"
ENDIF
lcString = lcString + IIF(RIGHT(lcString, 1) = "\", "", "\") + SYS(3)
lnHandle = FCREATE(lcString)
IF lnHandle >= 0
   = FCLOSE(lnHandle)
   ERASE (lcString)
ENDIF

RETURN (lnHandle >= 0)



*: EOF: ValidFN.PRG
